@@ -13,14 +13,17 @@ class Server:
         self.packagePrintBlacklist = []
         self.packagePrintBlacklist.append("ALIVE")
         self.packagePrintBlacklist.append("ALIVE_OK")
+        self.maxClients = 10
+        self.printUnsignedData = True
+        self.eventScope = "global"
 
-    def start(self, host="", port=0, maxClients=10, printUnsignedData=True):
+    def start(self, host="", port=0):
         if host == "":
             host = socket.gethostbyname(socket.gethostname())
         logger.log("§8[§eSERVER§8] §6Starting...")
         try:
             self.serverSocket.bind((host, port))
-            self.serverSocket.listen(maxClients)
+            self.serverSocket.listen(self.maxClients)
             logger.log("§8[§eSERVER§8] §aListening on §6" + str((host, port)))
         except Exception as e:
             logger.log("§8[§eSERVER§8] §8[§cERROR§8] §cFailed: " + str(e))
@@ -37,11 +40,11 @@ class Server:
                             if recvData.endswith("}" + self.seq):
                                 if lastData != "":
                                     recvData = lastData + recvData
-                                    if printUnsignedData:
+                                    if self.printUnsignedData:
                                         logger.log("§8[§eSERVER§8] §8[§cWARNING§8] §cUnsigned data repaired")
                         if not recvData.endswith("}" + self.seq):
                             lastData += recvData
-                            if printUnsignedData:
+                            if self.printUnsignedData:
                                 logger.log("§8[§eSERVER§8] §8[§cWARNING§8] §cReceiving unsigned data: §r" + recvData)
                             continue
                         if "}" + self.seq + "{" in recvData:
@@ -61,11 +64,15 @@ class Server:
                             if data["METHOD"] == "AUTHENTICATION":
                                 logger.log("§8[§eSERVER§8] §r[IN] " + data["METHOD"])
                                 if data["PASSWORD"] == self.password:
+                                    for c in self.clients:
+                                        if c["clientID"] == data["CLIENT_ID"]:
+                                            self.sendTo(clientSocket, {"METHOD": "AUTHENTICATION_FAILED"})
+                                            break
                                     client = {"clientSocket": clientSocket, "clientID": data["CLIENT_ID"], "clientType": data["CLIENT_TYPE"]}
                                     self.clients.append(client)
                                     self.sendTo(clientSocket, {"METHOD": "AUTHENTICATION_OK"})
                                     logger.log("§8[§eSERVER§8] §aClient '" + data["CLIENT_ID"] + "' authenticated")
-                                    events.call("ON_CLIENT_CONNECT", params=[client])
+                                    events.call("ON_CLIENT_CONNECT", params=[client], scope=self.eventScope)
                                 else:
                                     self.sendTo(clientSocket, {"METHOD": "AUTHENTICATION_FAILED"})
                                     break
@@ -74,7 +81,7 @@ class Server:
                                 if client is not None:
                                     if data["METHOD"] not in self.packagePrintBlacklist:
                                         logger.log("§8[§eSERVER§8] §r[IN] " + data["METHOD"])
-                                    events.call("ON_RECEIVE", [client, data])
+                                    events.call("ON_RECEIVE", params=[client, data], scope=self.eventScope)
                                 else:
                                     logger.log("§8[§eSERVER§8] §8[§cWARNING§8] §cReceiving not authenticated package: §r" + data["METHOD"])
                 except Exception as e:
@@ -83,7 +90,8 @@ class Server:
             self.clientSocks.remove(clientSocket)
             for client in self.clients:
                 if client["clientSocket"] == clientSocket:
-                    events.call("ON_CLIENT_DISCONNECT", params=[client])
+                    events.call("ON_CLIENT_DISCONNECT", params=[client], scope=self.eventScope)
+                    logger.log("§8[§eSERVER§8] §6Client '" + client["clientID"] + "' disconnected")
                     self.clients.remove(client)
             clientSocket.close()
             logger.log("§8[§eSERVER§8] §6Client disconnected")
@@ -115,7 +123,7 @@ class Server:
             if e == BrokenPipeError:
                 sock.close()
 
-    def sendToClient(self, clientID, data):
+    def sendToClientID(self, clientID, data):
         for client in self.clients:
             if client["clientID"] == clientID:
                 self.sendTo(clientID["clientSocket"], data)
